Imports System.IO

Public Class Form1

  Private sDataFilename As String
  Private bEditMode As Boolean
  Private bTextHasChanged As Boolean

  Private Sub cmdCalc_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCalc.Click
    Dim objLR As CStatSum
    Dim objRes As CResults
    Dim I, nDataCOunt As Integer
    Dim fShiftX, fShiftY, fScaleX, fScaleY As Double
    Dim sBuffer As String

    If sDataFilename = "" Then
      MessageBox.Show("Please select a data file first", "Error", MessageBoxButtons.OK, MessageBoxIcon.Hand)
      Exit Sub
    End If
    objLR = New CStatSum
    objRes = New CResults

    If bTextHasChanged Then
      If MessageBox.Show("Save changed data?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then
        File.WriteAllText(sDataFilename, txtRes.Text)
      End If
      bTextHasChanged = False
    End If
    bEditMode = False

    ' check the Shift X text box
    If txtShiftX.Text.Length > 0 Then
      fShiftX = Double.Parse(txtShiftX.Text)
    Else
      fShiftX = 0
    End If

    ' check the Shift Y text box
    If txtShiftY.Text.Length > 0 Then
      fShiftY = Double.Parse(txtShiftY.Text)
    Else
      fShiftY = 0
    End If

    ' check the Scale X text box
    If txtScaleX.Text.Length > 0 Then
      fScaleX = Double.Parse(txtScaleX.Text)
      If fScaleX = 0 Then fScaleX = 1
    Else
      fScaleX = 1
    End If

    ' check the Scale Y text box
    If txtScaleY.Text.Length > 0 Then
      fScaleY = Double.Parse(txtScaleY.Text)
      If fScaleY = 0 Then fScaleY = 1
    Else
      fScaleY = 1
    End If

    If objLR.GetData(sDataFilename, nDataCOunt, fShiftX, fShiftY, fScaleX, fScaleY) Then
      Cursor = Cursors.WaitCursor
      objLR.FindBestFit(objRes)
      objRes.SortResults()
      sBuffer = ""
      sBuffer = "Source Data File: " & sDataFilename & vbCrLf & vbCrLf
      sBuffer = sBuffer & "Date/Time: " & Now() & vbCrLf & vbCrLf
      sBuffer = sBuffer & "Number of observations = " & nDataCOunt & vbCrLf & vbCrLf
      If fScaleX <> 1 Then sBuffer = sBuffer & "Scale X = " & fScaleX & vbCrLf
      If fShiftX <> 0 Then sBuffer = sBuffer & "Shift X = " & fShiftX & vbCrLf
      If fScaleY <> 1 Then sBuffer = sBuffer & "Scale Y = " & fScaleY & vbCrLf
      If fShiftY <> 0 Then sBuffer = sBuffer & "Shift Y = " & fShiftY & vbCrLf & vbCrLf
      For I = 0 To objRes.Count - 1
        sBuffer = sBuffer & "R-Sqr = " & objRes.GetR2(I).ToString & vbCrLf
        sBuffer = sBuffer & "Model: " & objRes.GetModel(I) & vbCrLf
        sBuffer = sBuffer & "A  = " & objRes.GetIntercept(I).ToString & _
                      ", B = " & objRes.GetSlope(I).ToString & vbCrLf
      Next
      txtRes.Text = sBuffer
      sBuffer = ""
      Cursor = Cursors.Default
      cmdSaveRes.Enabled = True
    Else
      MessageBox.Show("Error in processing data", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
    End If


  End Sub

  Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    sDataFilename = ""
    cmdCalc.Enabled = False
    cmdSaveRes.Enabled = False
    bEditMode = False
  End Sub

  Private Sub cmdReadData_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdReadData.Click
    dlgReadData.Filter = "All files (*.*)|*.*|Text files|*.txt|Data files (*.dat)|*.dat"
    If dlgReadData.ShowDialog = Windows.Forms.DialogResult.OK Then
      sDataFilename = dlgReadData.FileName
      txtRes.Text = File.ReadAllText(sDataFilename)
      cmdCalc.Enabled = True
      cmdSaveRes.Enabled = True
      bTextHasChanged = False
      bEditMode = True
    End If
  End Sub

  Private Sub cmdSaveRes_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSaveRes.Click
    dlgSaveRes.Filter = "All files (*.*)|*.*|Text files|*.txt|Data files (*.dat)|*.dat"
    If dlgSaveRes.ShowDialog = Windows.Forms.DialogResult.OK Then
      File.WriteAllText(dlgSaveRes.FileName, txtRes.Text)
      bTextHasChanged = False
    End If
  End Sub

  Private Sub cmdClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClose.Click
    If MessageBox.Show("Close application?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.Yes Then
      Close()
    End If
  End Sub

  Private Sub txtRes_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtRes.TextChanged
    If bEditMode Then bTextHasChanged = True
  End Sub

  Private Sub cmdHelp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdHelp.Click
    Dim sText As String
    Dim I As Integer

    sText = "Each line can be 1) empty, 2) a comment line or 3) a data line" & vbCrLf & _
            "A data line has a pair of y and x values separated by a comma" & vbCrLf & _
             "A data line can have a weight value that is appended after x and is separated by a comma" & vbCrLf & _
            "Weights are optional and need to appear when their values are not 1" & vbCrLf & _
            "A comment line must NOT start with any of the chatacters +-.0123456789" & vbCrLf & _
            "Show an example?"
    If MessageBox.Show(sText, "Help", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = Windows.Forms.DialogResult.Yes Then
      bEditMode = False
      sText = "Sample data (example of free form comment line)" & vbCrLf & _
              "45,32" & vbCrLf & _
              "67,34" & vbCrLf & _
              "Next line is an observation that is temporaryly commente dout" & vbCrLf & _
              "! 56,23" & vbCrLf
      For I = 1 To 10
        sText = sText & Int(200 * Rnd(1)) & "," & Int(200 * Rnd(1)) & vbCrLf
      Next
      txtRes.Text = sText
    End If

  End Sub
End Class
